@@ -5,6 +5,7 @@ import uuid
 import re
 import datetime
 import pathlib
+import mimetypes
 import hashlib
 
 import django.utils
@@ -15,7 +16,7 @@ import mini_buildd.api
 
 LOG = logging.getLogger(__name__)
 
-register = django.template.Library()  # pylint: disable=invalid-name  # django needs this lower-case afaiu
+register = django.template.Library()
 
 
 class Css():
@@ -33,6 +34,7 @@ class Css():
 
     MAP = {
         "sbuild_status": {
+            "failed": ERROR,
             "attempted": ERROR,
             "given-back": ERROR,
             "skipped": INFO,
@@ -115,8 +117,8 @@ def mbd_basename(path):
 
 
 @register.simple_tag
-def mbd_join(path1, path2):
-    return os.path.join(path1, path2)
+def mbd_join(path, *paths):
+    return os.path.join(path, *paths)
 
 
 @register.filter
@@ -135,10 +137,20 @@ def mbd_hash(data):
     return f"mbd-hash-{hashlib.sha224(bytes(data, encoding=mini_buildd.config.CHAR_ENCODING)).hexdigest()}"
 
 
+@register.filter
+def mbd_codename(diststr):
+    """Get codename from diststr. On any error, just return empty string."""
+    try:
+        return mini_buildd.dist.Dist(diststr).codename
+    except Exception:
+        return ""
+
+
 @register.simple_tag
 def mbd_model_stats(model):
     ret = {}
-    model_class = eval(f"mini_buildd.models.{model}")  # pylint: disable=eval-used
+    mdl, _sep, cls = model.partition(".")  # model="daemon.Daemon" (see admin.py)
+    model_class = getattr(getattr(mini_buildd.mdls(), mdl), cls)
     if getattr(model_class, "mbd_is_prepared", None):
         # Status model
         ret["active"] = model_class.objects.filter(status__exact=model_class.STATUS_ACTIVE).count()
@@ -151,7 +163,7 @@ def mbd_model_stats(model):
 @register.simple_tag(takes_context=True)
 def mbd_next(context):
     request = context.get("request")
-    return request.get_full_path() if request.GET.get("output") != "snippet" else request.META.get("HTTP_REFERER")
+    return request.get_full_path() if request.GET.get("output") != "html-snippet" else request.META.get("HTTP_REFERER")
 
 
 @register.inclusion_tag("mini_buildd/includes/tags/accounts.html", takes_context=True)
@@ -166,7 +178,7 @@ def mbd_accounts(context, url_name, name=None, title=None, nxt=None, cls=""):
 
 
 @register.inclusion_tag("mini_buildd/includes/tags/api.html", takes_context=True)
-def mbd_api(context, call, name=None, title=None, hide_options=False, verbose=False, output="page", **kwargs):
+def mbd_api(context, call, name=None, hide_options=False, verbose=False, **kwargs):
     if isinstance(call, str):
         api_cls = mini_buildd.api.CALLS.get(call)
         if api_cls is None:
@@ -179,24 +191,18 @@ def mbd_api(context, call, name=None, title=None, hide_options=False, verbose=Fa
     args = {"mandatory": [], "optional": [], "hidden": []}
     for arg in api_call.args.values():
         args["mandatory" if arg.needs_value() else "hidden" if hide_options else "optional"].append(arg)
+    args["optional" if args["optional"] else "hidden"].insert(0, mini_buildd.api.Output())
 
     return {
         "request": context.get("request"),
         "args": args,
         "call": api_call,
         "name": api_call.name if name is None else name,
-        "title": api_call.doc if title is None else title,
         "is_authorized": api_call.AUTH.is_authorized(context.get("user")),
         "verbose": verbose,
-        "output": output,
     }
 
 
-@register.inclusion_tag("mini_buildd/includes/tags/api.html", takes_context=True)
-def mbd_api_popup(context, call, **kwargs):
-    return mbd_api(context, call, output="snippet", **kwargs)
-
-
 @register.inclusion_tag("mini_buildd/includes/tags/sbuild_status.html")
 def mbd_sbuild_status(bres):
     return {
@@ -209,13 +215,27 @@ def mbd_sbuild_status(bres):
 
 
 class PlainFilter(dict):
-    #: File extensions we display as 'plain' in the 1st place
-    PLAIN_FILES = [".buildlog", ".changes", ".json", ".dsc", ".buildinfo", ".log"]
+    #: Allow (non 'text/') mime types
+    ALLOWED_MIME_TYPES = ["application/json"]
+    #: Allow extensions that have no official mime type
+    ALLOWED_EXTENSIONS = ["buildlog", "changes", "dsc", "buildinfo", "log", "upload"]
+    #: Allow complete (base) file names
+    ALLOWED_FILES = ["InRelease", "Release", "Sources", "Packages", ".sbuildrc"]
+
+    def ok_to_filter(self, file_path):
+        mime_type = mimetypes.guess_type(file_path)[0]
+        name = os.path.basename(file_path)
+        return \
+            mime_type is not None and mime_type.startswith("text/") or \
+            mime_type in self.ALLOWED_MIME_TYPES or \
+            self["extension"] in self.ALLOWED_EXTENSIONS or \
+            name in self.ALLOWED_FILES
 
     #: Regex filters for ``*.buildlog``.
     BUILDLOG_INDEX_REGEXES = [
         (re.compile(r"\| (.*) \|"), 1, "mbd-info"),                                            # sbuild section headers
         (re.compile(r"^MINI_BUILDD: .+"), 0, "mbd-info"),                                      # mini-buildd banners (see files.py)
+        # (re.compile(rf"^/.+/{mini_buildd.sbuild.SETUP_D_DIR}/(.+_.+)"), 1, "mbd-info"),      # .. attention:: mini-buildd setup scripts -- alternative to the banner approach above (maybe nicer if we can also drop banner support)
         (re.compile(r".*make.*\*\*\*.*"), 0, "mbd-error"),                                     # make fatal error
         (re.compile(r"(^W(ARNING)?: .+)|(^W(ARN)?: .+)", re.IGNORECASE), 0, "mbd-warning"),    # warnings (lintian, others)
         (re.compile(r"(^E(RROR)?: .+)|(^E(RR)?: .+)", re.IGNORECASE), 0, "mbd-error"),         # errors (lintian, others)
@@ -269,9 +289,11 @@ class PlainFilter(dict):
         self["toc"] = []  # May be filled by filters (table of contents)
 
         if file_path is not None and os.path.exists(file_path):
-            dummy_, ext = os.path.splitext(file_path)
-            if ext in self.PLAIN_FILES:
-                self["content"] = getattr(self, "filter_" + ext[1:], self.filter_default)(file_path)
+            dummy_, extension = os.path.splitext(file_path)
+            self["extension"] = extension[1:]
+
+            if self.ok_to_filter(file_path):
+                self["content"] = getattr(self, "filter_" + self["extension"], self.filter_default)(file_path)
 
 
 @register.inclusion_tag("mini_buildd/includes/tags/file.html")
@@ -279,27 +301,15 @@ def mbd_file(content=None, file_path=None, uri=None, title=None):
     return PlainFilter(content, file_path, uri, title)
 
 
-@register.simple_tag
-def mbd_parse_distribution(repo, dist, suite):
-    name = repo.mbd_get_distribution_string(dist, suite)
-    return {"name": name,
-            "uri": mini_buildd.config.ROUTES["repositories"].uris["static"].join(repo.identity, "dists", name + "/"),
-            "mandatory_version_regex": repo.mbd_get_mandatory_version_regex(dist, suite),
-            "snapshots": repo.mbd_reprepro.get_snapshots(name),
-            "apt_build_sources_list": mini_buildd.rrpes(lambda dist, suite: repo.mbd_get_apt_build_sources_list(dist, suite).get(with_comment=True), dist, suite),
-            "apt_build_preferences": repo.mbd_get_apt_build_preferences(dist, suite)}
-
-
 @register.inclusion_tag("mini_buildd/includes/tags/datatable.html")
-def mbd_datatable(identity, length_menu="[[5, 10, 50, -1], [5, 10, 50, 'All']]", page_length=10, order=None, no_order="[]", searching="true", no_search="[]", paging="true", info="true"):
+def mbd_datatable(identity, length_menu="[[-1, 10, 20, 50], ['All', 10, 20, 50]]", page_length=20, order=None, columns="null", searching="true", paging="true", info="true"):
     return {
         "identity": identity,
         "length_menu": django.utils.safestring.mark_safe(length_menu),
         "page_length": page_length,
         "order": order,
-        "no_order": no_order,
+        "columns": columns,
         "searching": searching,
-        "no_search": no_search,
         "paging": paging,
         "info": info,
     }
