@@ -10,6 +10,7 @@ import gpg
 import gpg.errors
 import gpg.constants.sig.mode
 
+import mini_buildd
 import mini_buildd.config
 import mini_buildd.misc
 import mini_buildd.call
@@ -55,6 +56,26 @@ class Colons():
         return self._get(9)
 
 
+class GpgmeVerifyFailed(mini_buildd.HTTPUnauthorized):
+    @classmethod
+    def sig_info(cls, signature):
+        """Human-readable info combining actual keyid (if subkey, gpgme only provides fpr of subkey) and gpgme reason."""
+        try:
+            _key, info = mini_buildd.get_daemon().public_key_cache.keyinfo(signature.fpr)
+            return f"{gpg.errors.GpgError(signature.status).code_str}: {info['key']} ({','.join(info['subkeys'])}): {info['user']}"
+        except BaseException as e:
+            LOG.warning(f"Can't get keyinfo for gpgme signature: {e}")
+            return f"{signature}"
+
+    @classmethod
+    def sigs_info(cls, signatures):
+        return "Signatures: " + ", ".join([cls.sig_info(s) for s in signatures])
+
+    def __init__(self, detail, signatures):
+        self.signatures = signatures if signatures is not None else []
+        super().__init__(f"GPGME verify failed: {detail}: {self.sigs_info(self.signatures)}")
+
+
 class BaseGnuPG():
     @classmethod
     def get_flavor(cls):
@@ -83,7 +104,7 @@ class BaseGnuPG():
                         "--batch"]
         self.context = gpg.Context(home_dir=self.home)
 
-        LOG.info(f"GPG {self.flavor}: {self.gpg_cmd}")
+        LOG.debug(f"GPG {self.flavor}: {self.gpg_cmd}")
 
     def gen_secret_key(self, template):
         flavor_additions = {"2.1": "\n%no-protection\n",
@@ -164,39 +185,23 @@ class BaseGnuPG():
         except BaseException as e:
             raise mini_buildd.HTTPBadRequest("GnuPG authorization failed") from e
 
-    @classmethod
-    def gpgme_sig2info(cls, signature):
-        """Human-readable info combining actual keyid (if subkey, gpgme only provides fpr of subkey) and gpgme reason."""
-        try:
-            _key, info = PublicKeyCache().keyinfo(signature.fpr)
-            return f"{gpg.errors.GpgError(signature.status).code_str}: {info['key']} ({','.join(info['subkeys'])}): {info['user']}"
-        except BaseException as e:
-            LOG.warning(f"Can't get keyinfo for gpgme signature: {e}")
-            return f"{signature}"
-
-    @classmethod
-    def log_signatures(cls, signatures, key=None):
-        """Log signatures in human-readable info."""
-        for s in signatures:
-            LOG.info(f"SIGNATURE[{key}]: {cls.gpgme_sig2info(s)}")
-
     def gpgme_verify(self, signed_message, signature=None):
         try:
             return self.context.verify(signed_message.encode(mini_buildd.config.CHAR_ENCODING), signature)
         except gpg.errors.GpgError as e:
-            raise mini_buildd.HTTPUnauthorized(f"GPG verify failed: {e}")
+            raise GpgmeVerifyFailed(f"{e}", e.results.signatures if e.results is not None else None) from e
 
-    def verify_release(self, signed_data, signature=None, accept_expired=False, needs_all=False, log_key=None):
+    def gpgme_verify_release(self, signed_data, signature=None, accept_expired=False, needs_all=False):
         def verify(result):
-            accept = [gpg.errors.NO_ERROR] + [gpg.errors.KEY_EXPIRED] if accept_expired else []  # pylint: disable=no-member
+            accept = [gpg.errors.NO_ERROR] + [gpg.errors.KEY_EXPIRED] if accept_expired else []
             accepted, failed = [], []
             for s in result.signatures:
                 (accepted if gpg.errors.GpgError(s.status).code in accept else failed).append(s)
 
             if not accepted:
-                raise mini_buildd.HTTPUnauthorized(f"GPG verify failed (no accepted signature found): {[self.gpgme_sig2info(s) for s in result.signatures]}")
+                raise GpgmeVerifyFailed("No accepted signature found", result.signatures)
             if needs_all and failed:
-                raise mini_buildd.HTTPUnauthorized(f"GPG verify failed (needs all, but some signatures not accepted): {[self.gpgme_sig2info(s) for s in result.signatures]}")
+                raise GpgmeVerifyFailed("Needs all, but some signatures not accepted", result.signatures)
             return result.signatures
 
         signed_data.seek(0)
@@ -205,10 +210,11 @@ class BaseGnuPG():
 
         try:
             _dat, result = self.context.verify(signed_data, signature)
-            self.log_signatures(result.signatures, key=log_key)
             return verify(result)
         except (gpg.errors.BadSignatures, gpg.errors.MissingSignatures) as e:
             return verify(e.result)
+        except gpg.errors.GpgError as e:
+            raise GpgmeVerifyFailed(f"{e}", e.results.signatures if e.results is not None else None) from e
 
     def sign(self, file_name, identity=None):
         # 1st: copy the unsigned file and add an extra new line
@@ -225,7 +231,7 @@ class BaseGnuPG():
             if os.path.exists(signed_file):
                 os.remove(signed_file)
 
-        # Retrying sign call; workaround for mystery https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=849551
+        # .. error:: **compat** (:debbug:`849551`): Retrying sign call -- workaround for mystery bug
         mini_buildd.call.call_with_retry(self.gpg_cmd
                                          + ["--armor", "--textmode", "--clearsign", "--output", signed_file]
                                          + (["--local-user", identity] if identity else []) + [unsigned_file],
@@ -238,25 +244,30 @@ class BaseGnuPG():
         os.remove(unsigned_file)
 
     def gpgme_sign(self, message):
-        signed, _ = self.context.sign(message.encode(mini_buildd.config.CHAR_ENCODING), mode=gpg.constants.sig.mode.CLEAR)  # pylint: disable=no-member
+        signed, _ = self.context.sign(message.encode(mini_buildd.config.CHAR_ENCODING), mode=gpg.constants.sig.mode.CLEAR)
         return signed.decode(mini_buildd.config.CHAR_ENCODING)
 
 
 class GnuPG(BaseGnuPG):
+    def update(self):
+        self.pub_key = self.get_pub_key()
+        self.pub_key_id = self.get_first_sec_key()
+
     def __init__(self, template, fullname, email):
         super().__init__(home=mini_buildd.config.ROUTES["home"].path.join(".gnupg"))
         self.template = f"""{template}
 Name-Real: {fullname}
 Name-Email: {email}
 """
+        self.update()
 
     def prepare(self):
-        if not self.get_pub_key():
+        if not self.pub_key:
             LOG.info("Generating GnuPG secret key (this might take some time)...")
             self.gen_secret_key(self.template)
-            LOG.info("New GnuPG secret key prepared...")
+            self.update()
         else:
-            LOG.info("GnuPG key already prepared...")
+            LOG.debug("GnuPG key already prepared...")
 
     def remove(self):
         if os.path.exists(self.home):
@@ -303,10 +314,10 @@ class TmpGnuPG(BaseGnuPG, mini_buildd.misc.TmpDir):
 
     def __init__(self, tmpdir_options=None, **kwargs):
         mini_buildd.misc.TmpDir.__init__(self, **(tmpdir_options if tmpdir_options else {}))
-        super().__init__(home=self.tmpdir.name, **kwargs)
+        super().__init__(home=self.tmpdir, **kwargs)
 
 
-class PublicKeyCache(TmpGnuPG, metaclass=mini_buildd.misc.Singleton):
+class PublicKeyCache(TmpGnuPG):
     """
     Public key cache, initially with apt keys installed on the system (usually, from debian-archive-keyring package).
     """
@@ -318,7 +329,7 @@ class PublicKeyCache(TmpGnuPG, metaclass=mini_buildd.misc.Singleton):
         for keyring in glob.glob("/usr/share/keyrings/*-archive-*.gpg"):
             try:
                 mini_buildd.call.Call(self.gpg_cmd + ["--import", keyring]).check()
-                LOG.info(f"PublicKeyCache: Imported {keyring}")
+                LOG.debug(f"PublicKeyCache: Imported {keyring}")
             except BaseException as e:
                 LOG.warning(f"PublicKeyCache: Failed to import {keyring}: {e}")
 
@@ -343,7 +354,7 @@ class PublicKeyCache(TmpGnuPG, metaclass=mini_buildd.misc.Singleton):
                 if not keyserver:
                     raise Exception(f"Key '{key_id}' not in installed archive keys, and no keyserver configured.")
                 mini_buildd.call.call_with_retry(self.gpg_cmd + ["--armor", "--keyserver", keyserver, "--recv-keys", key_id], retry_max_tries=5, retry_sleep=5, timeout=self.KEYSERVER_RECV_TIMEOUT)
-                LOG.info(f"PublicKeyCache: '{key_id}' imported from key server '{keyserver}'.")
+                LOG.debug(f"PublicKeyCache: '{key_id}' imported from key server '{keyserver}'.")
 
             # Update pub keys
             pub_keys = self.get_pub_keys()
