@@ -9,9 +9,10 @@ import twisted.web.wsgi
 import twisted.web.static
 import twisted.web.resource
 import twisted.web.server
-# import twisted.web.util
 import twisted.logger
 import twisted.python.logfile
+import twisted.python.threadpool
+from twisted.internet import reactor
 
 import mini_buildd.config
 import mini_buildd.threads
@@ -37,15 +38,17 @@ class RootResource(twisted.web.resource.Resource):
         return self._wsgi_resource
 
 
+def mbd_is_ssl(request):
+    try:
+        return request.transport.__class__.__name__ == "TLSMemoryBIOProtocol"
+    except Exception as e:
+        LOG.warning(f"Hack to determine SSL failed: {e}")
+        return False
+
+
 class FileResource(twisted.web.static.File):
     """Twisted static resource enhanced with switchable index and regex matching support."""
 
-    DOC_MISSING_HTML = ("<html><body>"
-                        "<h1>Online Manual Not Available (<code>mini-buildd-doc</code> not installed?)</h1>"
-                        "Package <b><code>mini-buildd-doc</code></b> is not installed on this site (might be intentional to save space)."
-                        "<p><a href='/'>[Back]</a></p>"
-                        "</body></html>")
-
     def __init__(self, *args, mbd_uri=None, mbd_path=None, **kwargs):
         self.mbd_uri = mbd_uri
         self.mbd_path = mbd_path
@@ -55,9 +58,6 @@ class FileResource(twisted.web.static.File):
         else:
             super().__init__(*args, **kwargs)
 
-        if self.mbd_uri is not None and mbd_uri.with_doc_missing_error:
-            self.childNotFound = twisted.web.resource.NoResource(self.DOC_MISSING_HTML)
-
     def directoryListing(self):  # noqa (pep8 N802)
         if not self.mbd_uri.with_index:
             return self.forbidden
@@ -81,7 +81,30 @@ class FileResource(twisted.web.static.File):
         child.mbd_path = self.mbd_path  # getChild() implicitly calls our __init__ again. There seem to be no way to get around this monkey patch.
         return child
 
+    @classmethod
+    def mbd_producer_workaround(cls, request):
+        """
+        .. attention:: **compat** (``twisted``): Always unregister producer (avoids random static file error)
+
+        Avoids seemingly random::
+
+          RuntimeError: Cannot register producer <twisted.web.static.NoRangeStaticProducer object at 0xfoo>, because producer <twisted.internet._producer_helpers._PullToPush object at 0xbar> was never unregistered.
+
+        errors from twisted. This in turn randomly breaks ``apt update`` calls with (slightly misleading) 'size mismatch' errors.
+
+        Rare enough to never ever have shown in testsuite runs, but on bigger repos (with many (parallel?) apt calls), it's almost certain to appear.
+        """
+        what = cls.mbd_producer_workaround.__doc__.splitlines()[1].strip()
+        try:
+            # request.channel.unregisterProducer()
+            request.channel.loseConnection()
+        except BaseException as e:
+            mini_buildd.log_exception(LOG, what, e)
+
     def render(self, request):
+        if not mbd_is_ssl(request):
+            self.mbd_producer_workaround(request)
+
         if self.mbd_uri.cache_ttl:
             request.setHeader("Cache-Control", f"public, max-age={self.mbd_uri.cache_ttl}, no-transform")
         return super().render(request)
@@ -90,32 +113,36 @@ class FileResource(twisted.web.static.File):
 class Events(twisted.web.resource.Resource):
     @classmethod
     def _render(cls, request):
-        try:
-            LOG.debug(f"{request.channel}: Waiting for event...")
-            since = datetime.datetime.fromisoformat(request.args[b"since"][0].decode(mini_buildd.config.CHAR_ENCODING)) if b"since" in request.args else None
-            queue = mini_buildd.get_daemon().events_queue.attach(request.channel, since=mini_buildd.misc.Datetime.check_aware(since))
-            event = queue.get()
-            LOG.debug(f"{request.channel}: Got event: {event}")
-            if event is mini_buildd.config.SHUTDOWN:
-                raise mini_buildd.HTTPShutdown
-            request.write(json.dumps(event.to_json()).encode(mini_buildd.config.CHAR_ENCODING))
+        LOG.debug(f"{request.channel}: Waiting for event...")
+        after = datetime.datetime.fromisoformat(request.args[b"after"][0].decode(mini_buildd.config.CHAR_ENCODING)) if b"after" in request.args else None
+        queue = mini_buildd.get_daemon().events.attach(request.channel, after=mini_buildd.misc.Datetime.check_aware(after))
+        event = queue.get()
+        LOG.debug(f"{request.channel}: Got event: {event}")
+        if event is mini_buildd.config.SHUTDOWN:
+            raise mini_buildd.HTTPShutdown
+        request.write(json.dumps(event.to_json()).encode(mini_buildd.config.CHAR_ENCODING))
+        request.finish()
+
+    @classmethod
+    def _on_error(cls, failure, request):
+        http_exception = mini_buildd.e2http(failure.value)
+        request.setResponseCode(http_exception.status, f"{http_exception}".encode(mini_buildd.config.CHAR_ENCODING))
+        if getattr(request, "_disconnected", False):
+            request.notifyFinish()
+        else:
             request.finish()
-        except Exception as e:
-            http_exception = mini_buildd.e2http(e)
-            request.setResponseCode(500, f"{http_exception}".encode(mini_buildd.config.CHAR_ENCODING))
-            if getattr(request, "_disconnected", False):
-                request.notifyFinish()
-            else:
-                request.finish()
 
     @classmethod
     def mbd_ssl_workaround(cls, request):
         """
-        Twisted SSL timeout workaround (avoids spurious disconnects w/ SSL).
+        .. attention:: **compat** (``twisted < 22.2``): SSL timeout workaround (avoids spurious disconnects w/ SSL).
 
-        WTF: When using SSL, (event queue) connections spuriously disconnect (twisted: "Forcibly timing out client"), even
-        though no timeout has been specified (timeOut=None). Manually disabling 'abortTimeout' as well (which otherwise
-        defaults to 15 seconds) seems to fix the SSL case (and hopefully has no other bad effects).
+          WTF: When using SSL, (event queue) connections spuriously disconnect (twisted: "Forcibly timing out client"), even
+          though no timeout has been specified (timeOut=None). Manually disabling 'abortTimeout' as well (which otherwise
+          defaults to 15 seconds) seems to fix the SSL case (and hopefully has no other bad effects).
+
+          Retests with ``twisted 22.2`` could not (yet) reproduce this behaviour. So I guess it would be fine
+          to remove this once we can pimp the dependency in ``debian/control``.
         """
         what = cls.mbd_ssl_workaround.__doc__.splitlines()[1].strip()
         try:
@@ -124,17 +151,18 @@ class Events(twisted.web.resource.Resource):
         except BaseException as e:
             mini_buildd.log_exception(LOG, what, e)
 
-    def render_GET(self, request):  # pylint: disable=invalid-name  # noqa (pep8 N802)
-        self.mbd_ssl_workaround(request)
+    def render_GET(self, request):  # noqa (pep8 N802)
+        if mbd_is_ssl(request):
+            self.mbd_ssl_workaround(request)
         request.setHeader("Content-Type", f"application/json; charset={mini_buildd.config.CHAR_ENCODING}")
-        twisted.internet.threads.deferToThread(self._render, request)
+        twisted.internet.threads.deferToThread(self._render, request).addErrback(self._on_error, request)
         return twisted.web.server.NOT_DONE_YET
 
 
 class HttpD(mini_buildd.threads.Thread):
     def _add_route(self, uri, resource):
         """Add route from (possibly nested) path from config -- making sure already existing parent routes are re-used."""
-        LOG.info(f"Adding route: {uri} -> {resource}")
+        LOG.debug(f"Adding route: {uri} -> {resource}")
 
         uri_split = uri.twisted().split("/")
         res = self.resource
@@ -148,7 +176,7 @@ class HttpD(mini_buildd.threads.Thread):
             res = sub
         res.putChild(bytes(uri_split[-1], encoding=mini_buildd.config.CHAR_ENCODING), resource)
 
-    def __init__(self, wsgi_app):
+    def __init__(self, wsgi_app, minthreads=0, maxthreads=10):
         super().__init__(name="httpd")
         self.route_hierarchy = {}
 
@@ -156,7 +184,8 @@ class HttpD(mini_buildd.threads.Thread):
         twisted.logger.globalLogPublisher.addObserver(twisted.logger.STDLibLogObserver(name=__name__))
 
         # HTTP setup
-        self.resource = RootResource(twisted.web.wsgi.WSGIResource(mini_buildd.net.reactor(), mini_buildd.net.reactor().getThreadPool(), wsgi_app))  # pylint: disable=no-member
+        self.thread_pool = twisted.python.threadpool.ThreadPool(minthreads, maxthreads)
+        self.resource = RootResource(twisted.web.wsgi.WSGIResource(reactor, self.thread_pool, wsgi_app))
         self.site = Site(self.resource, logPath=mini_buildd.config.ROUTES["log"].path.join(mini_buildd.config.ACCESS_LOG_FILE))
 
         # Static routes
@@ -168,12 +197,22 @@ class HttpD(mini_buildd.threads.Thread):
         self._add_route(mini_buildd.config.ROUTES["events"].uris["attach"], Events())
 
         # Start sockets
+        ep_errors = []
+
+        def on_ep_error(f):
+            ep_errors.append(f.value)
+
         for ep in mini_buildd.config.HTTP_ENDPOINTS:
-            twisted.internet.endpoints.serverFromString(mini_buildd.net.reactor(), mini_buildd.net.ServerEndpoint(ep, protocol="http").description).listen(self.site)
+            twisted.internet.endpoints.serverFromString(reactor, mini_buildd.net.ServerEndpoint(ep, protocol="http").description).listen(self.site).addErrback(on_ep_error)
+
+        for e in ep_errors:
+            raise mini_buildd.HTTPUnavailable(str(e))
 
     def shutdown(self):
-        mini_buildd.net.reactor().stop()  # pylint: disable=no-member
+        self.thread_pool.stop()
+        reactor.stop()
 
     def run(self):
-        mini_buildd.net.reactor().run(installSignalHandlers=0)  # pylint: disable=no-member
+        self.thread_pool.start()
+        reactor.run(installSignalHandlers=0)
         LOG.debug(f"{self}: FINISHED")
