@@ -7,6 +7,7 @@ import tempfile
 import hashlib
 import re
 import logging
+import contextlib
 
 import dateparser
 
@@ -17,14 +18,6 @@ import mini_buildd.config
 LOG = logging.getLogger(__name__)
 
 
-def check_multiprocessing():
-    """Multiprocessing needs shared memory. This may be use to check for misconfigured shm early for better error handling."""
-    try:
-        multiprocessing.Lock()
-    except Exception as e:
-        raise Exception(f"multiprocessing not functional (shm misconfigured?): {e}") from e
-
-
 def singularize(s):
     """Singularize some english nouns from plural."""
     if s:
@@ -52,6 +45,10 @@ class Datetime:
     def from_stamp(cls, stamp):
         return datetime.datetime.fromtimestamp(stamp, tz=datetime.timezone.utc)
 
+    @classmethod
+    def from_iso(cls, isostr):
+        return datetime.datetime.fromisoformat(isostr).replace(tzinfo=datetime.timezone.utc)
+
     @classmethod
     def from_path(cls, path):
         return cls.from_stamp(os.path.getmtime(path))
@@ -85,7 +82,7 @@ class Datetime:
             raise mini_buildd.HTTPBadRequest(f"Date parse failed on: '{date_string}' ({Datetime.parse.__doc__})")
         if mini_buildd.misc.Datetime.is_naive(dt):
             dt = dt.astimezone()
-            LOG.info(f"Naive timestamp converted: {date_string} => {dt}")
+            LOG.debug(f"Naive timestamp converted: {date_string} => {dt}")
         return dt
 
 
@@ -162,31 +159,28 @@ class Singleton(type):
             del cls.__INSTANCES[cls]
 
 
-def skip_if_in_debug(key, func, *args, **kwargs):
-    if key in mini_buildd.config.DEBUG:
-        LOG.warning(f"DEBUG MODE('{key}'): Skipping: {func} {args} {kwargs}")
-    else:
-        func(*args, **kwargs)
-
-
-class TemporaryDirectory(tempfile.TemporaryDirectory):
-    """Like tempfile.TemporaryDirectory, but uses configured TMP path and honors 'keep' debug flag."""
-
-    def __init__(self, **kwargs):
-        super().__init__(dir=mini_buildd.config.ROUTES["tmp"].path.join(), **kwargs)
-
-    def cleanup(self):
-        skip_if_in_debug("keep", super().cleanup)
-
-
 class TmpDir():
-    """Unlike TemporaryDirectory, this may be used as mixin (use ```with contextlib.closing()...``` to work with actual object)."""
+    """Temporary dir class (use with ``with contextlib.closing()...``, or as mixin class)"""
 
     def __init__(self, **kwargs):
-        self.tmpdir = TemporaryDirectory(**kwargs)
+        tmp_route = mini_buildd.config.ROUTES.get("tmp")
+        self.tmpdir = tempfile.mkdtemp(dir=None if tmp_route is None else tmp_route.path.join(), **kwargs)
 
     def close(self):
-        skip_if_in_debug("keep", self.tmpdir.cleanup)
+        if "keep" in mini_buildd.config.DEBUG:
+            LOG.warning(f"DEBUG MODE('keep'): Skipping removal of temporary dir '{self.tmpdir}'")
+        else:
+            shutil.rmtree(self.tmpdir, onerror=lambda function, path, excinfo: LOG.warning(f"Could not remove temporary '{path}': {excinfo[1]}"))
+
+
+@contextlib.contextmanager
+def tmp_dir(**kwargs):
+    """Temporary dir context manager (use with ``with``)"""
+    tmpdir = TmpDir(**kwargs)
+    try:
+        yield tmpdir.tmpdir
+    finally:
+        tmpdir.close()
 
 
 def nop(*_args, **_kwargs):
@@ -297,29 +291,3 @@ def rmdirs(path):
     if os.path.exists(path):
         shutil.rmtree(path)
         LOG.info(f"Directory removed recursively: {path}")
-
-
-def clone_log(dst, src="mini_buildd"):
-    """Set up logger named 'dst' with the same handlers and loglevel as the logger named 'src'."""
-    src_log = logging.getLogger(src)
-    dst_log = logging.getLogger(dst)
-    dst_log.handlers = []
-    for h in src_log.handlers:
-        dst_log.addHandler(h)
-    dst_log.setLevel(src_log.getEffectiveLevel())
-
-
-def setup_console_logging(level=logging.DEBUG):
-    logging.addLevelName(logging.DEBUG, "D")
-    logging.addLevelName(logging.INFO, "I")
-    logging.addLevelName(logging.WARNING, "W")
-    logging.addLevelName(logging.ERROR, "E")
-    logging.addLevelName(logging.CRITICAL, "C")
-
-    ch = logging.StreamHandler()
-    ch.setFormatter(logging.Formatter("%(levelname)s: %(message)s"))
-
-    for ln in ["__main__", "mini_buildd"]:
-        log = logging.getLogger(ln)
-        log.addHandler(ch)
-        log.setLevel(level)
