@@ -53,6 +53,15 @@ from mini_buildd.builtins import mbd_cssmap
 LOG = logging.getLogger(__name__)
 
 
+def mbd_message(request, exception, prefix="", level=django.contrib.messages.ERROR, level_followups=django.contrib.messages.WARNING):
+    """Convey exception to user via django messaging, with extra multiline ('followups') formatting support"""
+    if request is not None:
+        for s in str(mini_buildd.e2http(exception)).split("\n"):
+            django.contrib.messages.add_message(request, level, django.utils.html.format_html(f"{prefix}{s}"))
+            level = level_followups
+            prefix = "↳ "
+
+
 class Model(django.db.models.Model):
     """
     Abstract father model for all mini-buildd models.
@@ -156,12 +165,10 @@ class Model(django.db.models.Model):
         if not re.match(regex, value):
             raise django.core.exceptions.ValidationError({field_name: f"Field does not match regex {regex}"})
 
-    @classmethod
-    def mbd_get_dependencies(cls):
+    def mbd_get_dependencies(self):
         return []
 
-    @classmethod
-    def mbd_get_reverse_dependencies(cls):
+    def mbd_get_reverse_dependencies(self):
         return []
 
     def save(self, *args, **kwargs):
@@ -353,13 +360,7 @@ class StatusModel(Model):
                 except BaseException as e:
                     msg = f"{action} failed: {o}"
                     mini_buildd.log_exception(LOG, msg, e)
-                    if request is not None:
-                        django.contrib.messages.add_message(request,
-                                                            django.contrib.messages.ERROR,
-                                                            django.utils.html.format_html("{}: {} (see <a href='{}daemon.log'>daemon.log</a>)",
-                                                                                          msg,
-                                                                                          str(mini_buildd.e2http(e)),
-                                                                                          mini_buildd.config.URIS["log"]["view"].uri))
+                    mbd_message(request, e, msg + ": ")
 
         @classmethod
         def mbd_actions(cls, request, queryset, actions):
