@@ -129,12 +129,13 @@ maintainers to be accidentally spammed.
     custom_hooks_directory = django.db.models.CharField(max_length=255, default="", blank=True, help_text="DEPRECATED, never used (there are now event hooks in predefined path ~/etc/event-hooks).")
 
     show_last_packages = django.db.models.IntegerField(
-        default=600,
-        help_text=MBD_HELP_NEEDS_RESTART + "Max number of events available.",
+        default=500,
+        help_text=MBD_HELP_NEEDS_RESTART + "Max number of items that can be stored.",
         verbose_name="Event Queue Size")
     show_last_builds = django.db.models.IntegerField(
         default=100,
-        help_text="DEPRECATED/UNUSED (since 2.0/events): Only show_last_packages is (mis)used as 'Event Queue Size'.")
+        help_text="Max number of items visible on home page (this essentially pimps its load time).",
+        verbose_name="Event Queue View Size")
 
     wait_for_build_results = django.db.models.IntegerField(
         default=5,
@@ -149,7 +150,7 @@ maintainers to be accidentally spammed.
         verbose_name_plural = "Daemon"
 
     class Admin(mini_buildd.models.base.StatusModel.Admin):
-        exclude = ("hostname", "custom_hooks_directory", "sbuild_jobs", "smtp_server", "show_last_builds", "wait_for_build_results", "keep_build_results")
+        exclude = ("hostname", "custom_hooks_directory", "sbuild_jobs", "smtp_server", "wait_for_build_results", "keep_build_results")
 
         filter_horizontal = ("notify",)
 
@@ -188,7 +189,7 @@ maintainers to be accidentally spammed.
 
     def mbd_setup_save(self, command_line):
         """Save setup command line."""
-        LOG.info(f"Saving setup: {command_line}")
+        LOG.debug(f"Saving setup: {command_line}")
         self.pickled_data = command_line
         self.save(update_fields=["pickled_data"])
 
@@ -222,8 +223,10 @@ maintainers to be accidentally spammed.
             raise django.core.exceptions.ValidationError("You can only create one Daemon instance!")
 
     def mbd_prepare(self):
-        self.mbd_gnupg().prepare()
-        LOG.info("Daemon GnuPG prepared.")
+        gnupg = self.mbd_gnupg()
+        gnupg.prepare()
+        mini_buildd.get_daemon().gnupg = gnupg
+        LOG.debug("Daemon GnuPG prepared.")
 
     @classmethod
     def mbd_sync(cls):
@@ -231,7 +234,7 @@ maintainers to be accidentally spammed.
 
     def mbd_remove(self):
         self.mbd_gnupg().remove()
-        LOG.info("Daemon GnuPG removed.")
+        LOG.debug("Daemon GnuPG removed.")
 
     def mbd_get_dependencies(self):
         """All active or to-be active repositories, remotes and chroots."""
@@ -248,10 +251,7 @@ maintainers to be accidentally spammed.
             LOG.warning("No active chroot or repository.")
 
     def mbd_get_archive_origin(self):
-        return self.mbd_get_extra_option("X-Archive-Origin", f"Mini-Buildd {self.identity}")
-
-    def mbd_get_pub_key(self):
-        return self.mbd_gnupg().get_pub_key()
+        return self.mbd_get_extra_option("X-Archive-Origin", f"Mini-Buildd archive {self.identity} on {mini_buildd.config.HOSTNAME_FQDN}")
 
     def mbd_get_dput_conf(self):
         ftp = self.mbd_get_ftp_endpoint()
@@ -279,7 +279,7 @@ incoming = /incoming
     def mbd_notify_event(self, event):
         repository, distribution = None, None
         try:
-            _dist, repository, distribution, _suite = mini_buildd.models.repository.parse_diststr(event.distribution)
+            repository, distribution, _suite = mini_buildd.models.repository.parse_diststr(event.distribution)
         except BaseException:
             pass
 
@@ -301,9 +301,9 @@ incoming = /incoming
                     else:
                         m_to.append((subject_prefix + subject, body + self._mbd_notify_signature(typ), self.email_address, [address]))
                         m_to_raw.append(address_raw)
-                        LOG.info(f"Notify: Adding {typ} address: {address}")
+                        LOG.debug(f"Notify: Adding {typ} address: {address}")
                 else:
-                    LOG.warning(f"Notify: Skipping {typ} address: {address}: Not allowed (only '{self.allow_emails_to}')")
+                    LOG.info(f"Notify: Skipping {typ} address: {address}: Not allowed (only '{self.allow_emails_to}')")
 
             def get_subscriptions():
                 source = "" if event.source is None else event.source
@@ -316,12 +316,12 @@ incoming = /incoming
 
             # Add hardcoded addresses from daemon
             for m in self.notify.all():
-                add_to(m.__str__(), "daemon", is_automatic=False)
+                add_to(str(m), "daemon", is_automatic=False)
 
             # Add hardcoded addresses from repository
             if repository:
                 for m in repository.notify.all():
-                    add_to(m.__str__(), "repository", is_automatic=False)
+                    add_to(str(m), "repository", is_automatic=False)
 
             # Add package uploader (Changed-By): Add even if we do not have a repo, so uploader is informed on such error cases too.
             if event.extra.get("changed_by") and (repository is None or repository.notify_changed_by):
