@@ -96,6 +96,17 @@ Example:
     def __str__(self):
         return f"{self.identity}: {' '.join([d.base_source.codename + ('' if d.mbd_is_active() else '*') for d in self.mbd_sorted_distributions()])}"
 
+    def mbd_json(self):
+        return {
+            "codenames": [d.base_source.codename for d in self.distributions.all()],
+            "layout": {
+                s.suite.name: {
+                    "uploadable": s.uploadable,
+                    "experimental": s.experimental,
+                    "migrates_to": s.migrates_to.suite.name if s.migrates_to else None,
+                } for s in self.layout.suiteoption_set.all()},
+        }
+
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.mbd_path = mini_buildd.config.ROUTES["repositories"].path.join(self.identity)
@@ -105,44 +116,48 @@ Example:
         super().clean(*args, **kwargs)
         self.mbd_validate_regex(r"^[a-z0-9]+$", self.identity, "identity")
 
+    def mbd_get_diststr(self, distribution, suite, rollback=None):
+        diststr = f"{distribution.base_source.codename}-{self.identity}-{suite.suite.name}"
+
+        if rollback is not None:
+            if rollback not in list(range(suite.rollback)):
+                raise mini_buildd.HTTPBadRequest(f"{diststr}: Rollback number out of range: {rollback} ({list(range(suite.rollback))})")
+            diststr += f"-rollback{rollback}"
+
+        return diststr
+
+    def mbd_get_diststrs(self, frollbacks=None, **suiteoption_filter):
+        """List of all distribution strings (except rollbacks) in this repo, optionally matching a suite options filter (unstable, experimental,...)"""
+        diststrs = []
+        for d in self.distributions.all():
+            for s in self.layout.suiteoption_set.filter(**suiteoption_filter):
+                diststrs.append(self.mbd_get_diststr(d, s))
+                if frollbacks is not None:
+                    for r in frollbacks(s):
+                        diststrs.append(self.mbd_get_diststr(d, s, rollback=r))
+        return diststrs
+
     def mbd_get_meta_distributions(self, distribution, suite_option):
         try:
             result = []
             for p in self.layout.mbd_get_extra_option("Meta-Distributions", "").split():
                 meta, d = p.split("=")
                 dist = f"{d.split('-')[0]}-{self.identity}-{d.split('-')[1]}"
-                if dist == self.mbd_get_distribution_string(distribution, suite_option):
+                if dist == self.mbd_get_diststr(distribution, suite_option):
                     result.append(meta)
             return result
         except BaseException as e:
             raise mini_buildd.HTTPBadRequest(f"Please fix syntax error in extra option 'Meta-Distributions' in layout '{self.layout}'") from e
 
-    def mbd_get_distribution_string(self, distribution, suite, rollback=None):
-        dist_string = f"{distribution.base_source.codename}-{self.identity}-{suite.suite.name}"
-
-        if rollback is not None:
-            if rollback not in list(range(suite.rollback)):
-                raise mini_buildd.HTTPBadRequest(f"{dist_string}: Rollback number out of range: {rollback} ({list(range(suite.rollback))})")
-            dist_string += f"-rollback{rollback}"
-
-        return dist_string
-
     def mbd_get_apt_pin(self, distribution, suite):
-        return f"release n={self.mbd_get_distribution_string(distribution, suite)}, o={mini_buildd.get_daemon().model.mbd_get_archive_origin()}"
+        return f"release n={self.mbd_get_diststr(distribution, suite)}, o={mini_buildd.get_daemon().model.mbd_get_archive_origin()}"
 
     def mbd_get_apt_preferences(self, distribution, suite, prio=1):
         return f"Package: *\nPin: {self.mbd_get_apt_pin(distribution, suite)}\nPin-Priority: {prio}\n"
 
-    def mbd_distribution_strings(self, **suiteoption_filter):
-        """Return a list with all full distributions strings, optionally matching a suite options filter (unstable, experimental,...)."""
-        result = []
-        for d in self.mbd_sorted_distributions():
-            result += [self.mbd_get_distribution_string(d, s) for s in self.layout.suiteoption_set.filter(**suiteoption_filter)]
-        return result
-
     @classmethod
     def mbd_get_apt_keys(cls, distribution):
-        result = mini_buildd.get_daemon().model.mbd_get_pub_key()
+        result = mini_buildd.get_daemon().gnupg.pub_key
         for e in distribution.extra_sources.all():
             for k in e.source.apt_keys.all():
                 result += k.key
@@ -167,10 +182,6 @@ Example:
 
         return result
 
-    def mbd_get_distribution_strings(self):
-        """List of all distribution strings (except rollbacks) in this repo."""
-        return [self.mbd_get_distribution_string(d, s) for d in self.distributions.all() for s in self.layout.suiteoption_set.all()]
-
     @classmethod
     def __mbd_subst_placeholders(cls, value, repository, distribution):
         return mini_buildd.misc.subst_placeholders(
@@ -189,18 +200,18 @@ Example:
             self, distribution)
 
     def mbd_get_apt_line(self, distribution, suite_option, rollback=None, snapshot=None):
-        dist_str = self.mbd_get_distribution_string(distribution, suite_option, rollback=rollback)
+        diststr = self.mbd_get_diststr(distribution, suite_option, rollback=rollback)
         if snapshot:
-            snapshots = self.mbd_reprepro.get_snapshots(dist_str)
+            snapshots = self.mbd_reprepro.get_snapshots(diststr)
             if snapshot not in snapshots:
-                raise mini_buildd.HTTPBadRequest(f"No such snapshot for {dist_str}: '{snapshot}' (available: {','.join(snapshots)})")
-            dist_str += os.path.join("/", "snapshots", snapshot)
+                raise mini_buildd.HTTPBadRequest(f"No such snapshot for {diststr}: '{snapshot}' (available: {','.join(snapshots)})")
+            diststr += os.path.join("/", "snapshots", snapshot)
 
         return mini_buildd.files.AptLine(mini_buildd.config.URIS["repositories"]["static"].url_join(),
                                          self.identity,
-                                         dist_str,
+                                         diststr,
                                          distribution.components.all(),
-                                         comment=f"{mini_buildd.get_daemon().model.mbd_get_archive_origin()} '{dist_str}': {self.mbd_get_apt_pin(distribution, suite_option)}")
+                                         comment=f"{mini_buildd.get_daemon().model.mbd_get_archive_origin()} '{diststr}': {self.mbd_get_apt_pin(distribution, suite_option)}")
 
     def mbd_get_apt_build_sources_list(self, distribution, suite_option):
         sources_list = mini_buildd.files.SourcesList()
@@ -253,23 +264,23 @@ Example:
         for d in self.distributions.all():
             for s in self.layout.suiteoption_set.all():
                 result += dist_template.format(
-                    distribution=self.mbd_get_distribution_string(d, s),
+                    distribution=self.mbd_get_diststr(d, s),
                     meta_distributions=" ".join(self.mbd_get_meta_distributions(d, s)),
                     origin=mini_buildd.get_daemon().model.mbd_get_archive_origin(),
                     components=" ".join(d.mbd_get_components()),
                     architectures=" ".join([x.name for x in d.architectures.all()]),
-                    desc=self.mbd_get_distribution_string(d, s),
+                    desc=self.mbd_get_diststr(d, s),
                     na="yes" if s.not_automatic else "no",
                     bau="yes" if s.but_automatic_upgrades else "no")
 
                 for r in range(s.rollback):
                     result += dist_template.format(
-                        distribution=self.mbd_get_distribution_string(d, s, r),
+                        distribution=self.mbd_get_diststr(d, s, r),
                         meta_distributions="",
                         origin=mini_buildd.get_daemon().model.mbd_get_archive_origin(),
                         components=" ".join(d.mbd_get_components()),
                         architectures=" ".join([x.name for x in d.architectures.all()]),
-                        desc=self.mbd_get_distribution_string(d, s, r),
+                        desc=self.mbd_get_diststr(d, s, r),
                         na="yes",
                         bau="no")
 
@@ -278,125 +289,117 @@ Example:
     def mbd_reprepro_update_config(self):
         if mini_buildd.files.Dir(os.path.join(self.mbd_path, "conf")).add(self._mbd_reprepro_conf_options()) \
                                                                      .add(self._mbd_reprepro_conf_distributions()).update():
-            self.mbd_reprepro.reindex(self.mbd_get_distribution_strings())
+            self.mbd_reprepro.reindex(self.mbd_get_diststrs())
 
-    def mbd_find_dsc_path(self, distribution, source, version, components=None):
+    def mbd_dsc_pool_path(self, source, version, raise_exception=True):
         """Get DSC pool path of an installed source (``<repo>/pool/...``)."""
-        subdir = source[:4] if source.startswith("lib") else source[0]
+        repositories_path = mini_buildd.config.ROUTES["repositories"].path
+        dscs = glob.glob(repositories_path.join(self.identity, "pool", "*", "*", source, mini_buildd.files.DebianName(source, version).dsc()))
 
-        for component in [c.name for c in sorted(distribution.components.all())] if components is None else components:
-            dsc = f"{self.identity}/pool/{component}/{subdir}/{source}/{mini_buildd.files.DebianName(source, version).dsc()}"
-            route = mini_buildd.config.ROUTES["repositories"]
-            if os.path.exists(route.path.join(dsc)):
-                LOG.debug(f"Found DSC in pool: {dsc}")
-                return component, dsc
+        LOG.debug(f"Repository {self.identity}: Found pool DSCs for '{source}-{version}': {dscs}")
 
-        # Not found in pool
-        return None, None
+        if len(dscs) > 1:  # This should not really ever happen. Pool-wise it could however with different components (main, contrib, etc)
+            LOG.warning(f"Found multiple DSCs for '{source}-{version}' in pool (only the 1st found will be used)")
+            for d in dscs:
+                LOG.warning(f"↳ {d}")
+
+        if len(dscs) < 1:
+            msg = f"Can't find DSC for '{source}-{version}' in pool of repository '{self.identity}'"
+            if raise_exception:
+                raise mini_buildd.HTTPBadRequest(msg)
+            LOG.warning(msg)
+            return None
+
+        return repositories_path.removeprefix(dscs[0])
 
     def _mbd_package_shift_rollbacks(self, distribution, suite_option, package_name):
-        reprepro_output = ""
-        for r in range(suite_option.rollback - 1, -1, -1):
-            src = self.mbd_get_distribution_string(distribution, suite_option, None if r == 0 else r - 1)
-            dst = self.mbd_get_distribution_string(distribution, suite_option, r)
-            LOG.info(f"Rollback: Moving {package_name}: {src} to {dst}")
-            try:
-                reprepro_output += self.mbd_reprepro.migrate(package_name, src, dst)
-            except BaseException as e:
-                mini_buildd.log_exception(LOG, "Rollback failed (ignoring)", e)
-        return reprepro_output
+        diststr = self.mbd_get_diststr(distribution, suite_option)
+        ls = self.mbd_reprepro.ls(package_name)
+        if ls.filter_rollbacks(version=ls[diststr]["version"]):
+            LOG.info(f"{package_name}_{ls[diststr]['version']}: Already in rollbacks, skipping rollback")
+        else:
+            for r in range(suite_option.rollback - 1, -1, -1):
+                src = self.mbd_get_diststr(distribution, suite_option, None if r == 0 else r - 1)
+                dst = self.mbd_get_diststr(distribution, suite_option, r)
+                LOG.info(f"Rollback: Moving {package_name}: {src} to {dst}")
+                try:
+                    self.mbd_reprepro.migrate(package_name, src, dst)
+                except BaseException as e:
+                    mini_buildd.log_exception(LOG, "Rollback failed (ignoring)", e)
 
-    def _mbd_package_migrate(self, package, distribution, suite, rollback=None, version=None, log_event=False):
-        reprepro_output = ""
-
-        src_dist = self.mbd_get_distribution_string(distribution, suite)
-        dst_dist = None
-        pkg_show = list(self.mbd_reprepro.ils(package))
-        src_pkg = None
+    def _mbd_package_migrate(self, package, distribution, suite, rollback=None, version=None, log_event=True):
+        src_diststr = self.mbd_get_diststr(distribution, suite)
+        dst_diststr = None
+        pkg_ls = self.mbd_reprepro.ls(package)
 
         if rollback is not None:
-            dst_dist = src_dist
-            LOG.info(f"Rollback restore of '{package}' from rollback {rollback} to '{dst_dist}'")
-            if self.mbd_reprepro.find_in(pkg_show, distribution=dst_dist):
-                raise mini_buildd.HTTPBadRequest(f"Package '{package}' exists in '{dst_dist}': Remove first to restore rollback")
+            dst_diststr = src_diststr
+            LOG.info(f"Rollback restore of '{package}' from rollback {rollback} to '{dst_diststr}'")
 
-            rob_dist = self.mbd_get_distribution_string(distribution, suite, rollback=rollback)
-            src_pkg = self.mbd_reprepro.find_in(pkg_show, distribution=rob_dist, version=version)
-            if src_pkg is None:
-                raise mini_buildd.HTTPBadRequest(f"Package '{package} ({'*' if version is None else version})' has no such version in rollback '{rollback}'")
+            pkg_ls.filter(diststr=dst_diststr, raise_if_found=f"Package '{package}' exists in '{dst_diststr}': Remove first to restore rollback")
+            rob_diststr = self.mbd_get_diststr(distribution, suite, rollback=rollback)
+            pkg_ls.filter(diststr=rob_diststr, version=version, raise_if_not_found="No such rollback source")
 
             # Actually migrate package in reprepro
-            reprepro_output += self.mbd_reprepro.migrate(package, rob_dist, dst_dist, version)
+            self.mbd_reprepro.migrate(package, rob_diststr, dst_diststr, version)
         else:
             # Get src and dst dist strings, and check we are configured to migrate
             if not suite.migrates_to:
-                raise mini_buildd.HTTPBadRequest(f"You can't migrate from '{src_dist}'")
-            dst_dist = self.mbd_get_distribution_string(distribution, suite.migrates_to)
+                raise mini_buildd.HTTPBadRequest(f"You can't migrate from '{src_diststr}'")
+            dst_diststr = self.mbd_get_diststr(distribution, suite.migrates_to)
 
             # Check if package is in src_dst
-            src_pkg = self.mbd_reprepro.find_in(pkg_show, distribution=src_dist, version=version)
-            if src_pkg is None:
-                raise mini_buildd.HTTPBadRequest(f"Package '{package} ({'*' if version is None else version})' not in '{src_dist}'")
+            pkg_ls.filter(diststr=src_diststr, version=version, raise_if_not_found="No such source")
 
             # Check that version is not already migrated
-            dst_pkg = self.mbd_reprepro.find_in(pkg_show, distribution=dst_dist)
-            if dst_pkg is not None and src_pkg["version"] == dst_pkg["version"]:
-                raise mini_buildd.HTTPBadRequest(f"Version '{src_pkg['version']}' already migrated to '{dst_dist}'")
+            if version is not None and pkg_ls.filter(diststr=dst_diststr, version=version):
+                raise mini_buildd.HTTPBadRequest(f"Version '{version}' already migrated to '{dst_diststr}'")
 
             # Shift rollbacks in the destination distributions
-            if dst_pkg is not None:
-                reprepro_output += self._mbd_package_shift_rollbacks(distribution, suite.migrates_to, package)
+            if pkg_ls.filter(diststr=dst_diststr):
+                self._mbd_package_shift_rollbacks(distribution, suite.migrates_to, package)
 
             # Actually migrate package in reprepro
-            reprepro_output += self.mbd_reprepro.migrate(package, src_dist, dst_dist, version)
+            self.mbd_reprepro.migrate(package, src_diststr, dst_diststr, version)
 
         if log_event:
-            mini_buildd.get_daemon().events_queue.log(mini_buildd.events.Type.MIGRATED, mini_buildd.changes.Changes({**src_pkg, **{"distribution": dst_dist}}, create_events=True))
-
-        return reprepro_output
+            pkg_ls = self.mbd_reprepro.ls(package)
+            mini_buildd.get_daemon().events.log(mini_buildd.events.Type.MIGRATED, pkg_ls.changes(dst_diststr, None).create_events())
 
     def mbd_package_migrate(self, package, distribution, suite, full=False, rollback=None, version=None):
-        reprepro_output = ""
         if full:
             while suite.migrates_to is not None:
-                reprepro_output += self._mbd_package_migrate(package, distribution, suite, rollback=rollback, version=version, log_event=True)
+                self._mbd_package_migrate(package, distribution, suite, rollback=rollback, version=version)
                 suite = suite.migrates_to
         else:
-            reprepro_output = self._mbd_package_migrate(package, distribution, suite, rollback=rollback, version=version, log_event=True)
-
-        return reprepro_output
+            self._mbd_package_migrate(package, distribution, suite, rollback=rollback, version=version)
 
     def mbd_package_remove(self, package, distribution, suite, rollback=None, version=None, without_rollback=False):
-        reprepro_output = ""
-
-        dist_str = self.mbd_get_distribution_string(distribution, suite, rollback)
-        src_pkg = self.mbd_reprepro.find_in(self.mbd_reprepro.ils(package), distribution=dist_str, version=version)
-        if not src_pkg:
-            raise mini_buildd.HTTPBadRequest(f"Package '{package}' not in '{dist_str}'")
+        diststr = self.mbd_get_diststr(distribution, suite, rollback)
+        src_pkg = self.mbd_reprepro.ls(package)
+        src_pkg.filter(diststr=diststr, version=version, raise_if_not_found="No such source")
 
         if rollback is None:
             if not without_rollback:
-                reprepro_output += self._mbd_package_shift_rollbacks(distribution, suite, package)
+                self._mbd_package_shift_rollbacks(distribution, suite, package)
             # Remove package
-            reprepro_output += self.mbd_reprepro.remove(package, dist_str, version)
+            self.mbd_reprepro.remove(package, diststr, version)
         else:
             # Rollback removal
-            reprepro_output += self.mbd_reprepro.remove(package, dist_str, version)
+            self.mbd_reprepro.remove(package, diststr, version)
 
             # Fix up empty rollback dist
             for r in range(rollback, suite.rollback - 1):
-                src = self.mbd_get_distribution_string(distribution, suite, r + 1)
-                dst = self.mbd_get_distribution_string(distribution, suite, r)
+                src_diststr = self.mbd_get_diststr(distribution, suite, r + 1)
+                dst_diststr = self.mbd_get_diststr(distribution, suite, r)
                 try:
-                    reprepro_output += self.mbd_reprepro.migrate(package, src, dst)
-                    reprepro_output += self.mbd_reprepro.remove(package, src)
+                    self.mbd_reprepro.migrate(package, src_diststr, dst_diststr)
+                    self.mbd_reprepro.remove(package, src_diststr)
                 except BaseException as e:
-                    mini_buildd.log_exception(LOG, f"Rollback: Moving '{package}' from '{src}' to '{dst}' FAILED (ignoring)", e, logging.WARN)
+                    mini_buildd.log_exception(LOG, f"Rollback: Moving '{package}' from '{src_diststr}' to '{dst_diststr}' FAILED (ignoring)", e, logging.WARN)
 
         # Notify
-        mini_buildd.get_daemon().events_queue.log(mini_buildd.events.Type.REMOVED, mini_buildd.changes.Changes(src_pkg, create_events=True))
-
-        return reprepro_output
+        mini_buildd.get_daemon().events.log(mini_buildd.events.Type.REMOVED, src_pkg.changes(diststr, None).create_events())
 
     def mbd_package_precheck(self, distribution, suite_option, package, version):
         # 1st, check that the given version matches the distribution's version restrictions
@@ -404,33 +407,33 @@ Example:
         if not re.compile(mandatory_regex).search(version):
             raise mini_buildd.HTTPBadRequest(f"Version restrictions failed for suite '{suite_option.suite.name}': '{mandatory_regex}' not in '{version}'")
 
-        pkg_show = list(self.mbd_reprepro.ils(package))
-        dist_str = self.mbd_get_distribution_string(distribution, suite_option)
+        pkg_ls = self.mbd_reprepro.ls(package)
+        diststr = self.mbd_get_diststr(distribution, suite_option)
 
         # 2nd: Check whether the very same version is already in any distribution
-        pkg_version_in_repo = self.mbd_reprepro.find_in(pkg_show, version=version)
-        if pkg_version_in_repo:
-            raise mini_buildd.HTTPBadRequest(f"Package '{package}' with same version '{version}' already installed in '{pkg_version_in_repo['distribution']}'")
+        pkg_ls.filter(version=version, raise_if_found="Source already installed")
 
         # 3rd: Check that distribution's current version is smaller than the to-be installed version
-        pkg_in_dist = self.mbd_reprepro.find_in(pkg_show, distribution=dist_str)
-        if pkg_in_dist and debian.debian_support.Version(version) < debian.debian_support.Version(pkg_in_dist["version"]):
-            raise mini_buildd.HTTPBadRequest(f"Package '{package}' has greater version '{pkg_in_dist['version']}' installed in '{dist_str}'")
+        pkg_in_dist = pkg_ls.filter(diststr=diststr)
+        if pkg_in_dist:
+            pkg_in_dist_version = pkg_in_dist[diststr]["version"]
+            if debian.debian_support.Version(version) < debian.debian_support.Version(pkg_in_dist_version):
+                raise mini_buildd.HTTPBadRequest(f"Package '{package}' has greater version '{pkg_in_dist_version}' installed in '{diststr}'")
 
-    def _mbd_buildresult_install(self, buildresult, dist_str):
+    def _mbd_buildresult_install(self, buildresult, diststr):
         # Don't try install if skipped
         if buildresult.cget("Sbuild-Status") == "skipped":
-            LOG.info(f"Skipped: {buildresult}")
+            LOG.debug(f"Skipped: {buildresult}")
         else:
-            with mini_buildd.misc.TemporaryDirectory(prefix="buildresult-untar-") as tmpdir:
+            with mini_buildd.misc.tmp_dir(prefix="buildresult-untar-") as tmpdir:
                 buildresult.untar(tmpdir)
-                self.mbd_reprepro.install(" ".join(glob.glob(os.path.join(tmpdir, "*.changes"))), dist_str)
+                self.mbd_reprepro.install(" ".join(glob.glob(os.path.join(tmpdir, "*.changes"))), diststr)
                 LOG.info(f"Installed: {buildresult}")
 
     def mbd_package_install(self, distribution, suite_option, changes, buildresults):
         """Install a dict arch:buildresult of successful build results."""
         # Get the full distribution str
-        dist_str = self.mbd_get_distribution_string(distribution, suite_option)
+        diststr = self.mbd_get_diststr(distribution, suite_option)
 
         # Check that all mandatory archs are present
         missing_mandatory_archs = [arch for arch in distribution.mbd_get_mandatory_architectures() if arch not in buildresults]
@@ -442,26 +445,26 @@ Example:
         LOG.debug(f"Package install: Package={package}")
 
         # Shift current package up in the rollback distributions (unless this is the initial install)
-        is_installed = self.mbd_reprepro.find_in(self.mbd_reprepro.ils(package), distribution=dist_str)
+        is_installed = self.mbd_reprepro.ls(package).filter(diststr=diststr)
         if is_installed:
             self._mbd_package_shift_rollbacks(distribution, suite_option, package)
 
         # First, install the dsc
-        self.mbd_reprepro.install_dsc(changes.dsc_file_path(), dist_str)
+        self.mbd_reprepro.install_dsc(changes.dsc_file_path(), diststr)
 
         # Second, install all build results; if any buildresult fails to install, rollback changes to repository
         try:
             for buildresult in list(buildresults.values()):
-                self._mbd_buildresult_install(buildresult, dist_str)
+                self._mbd_buildresult_install(buildresult, diststr)
         except Exception as e:
-            self.mbd_reprepro.remove(package, dist_str)
+            self.mbd_reprepro.remove(package, diststr)
             if is_installed:
-                self._mbd_package_migrate(package, distribution, suite_option, rollback=0)
+                self._mbd_package_migrate(package, distribution, suite_option, rollback=0, log_event=False)
             mini_buildd.log_exception(LOG, "Binary install failed", e)
             raise mini_buildd.HTTPInternal(f"Binary install failed for {package}: {mini_buildd.e2http(e)}")
 
     def mbd_sorted_distributions(self):
-        return sorted(self.distributions.all(), reverse=True)
+        return sorted(self.distributions.all())
 
     def mbd_icodenames(self):
         for d in self.mbd_sorted_distributions():
@@ -510,27 +513,6 @@ Example:
             result += [e.source for e in d.extra_sources.all()]
         return result
 
-    def mbd_parse_dist(self, dist, check_uploadable=False):
-        distribution = self.distributions.all().filter(base_source__codename__exact=dist.codename).first()
-        if distribution is None:
-            raise mini_buildd.HTTPBadRequest(f"No distribution for codename '{dist.codename}' in repository '{self.identity}'")
-
-        suite = self.layout.suiteoption_set.filter(suite__name=dist.suite).first()
-        if suite is None:
-            raise mini_buildd.HTTPBadRequest(f"No distribution for suite '{dist.suite}' in repository '{self.identity}'")
-
-        if check_uploadable:
-            if dist.rollback_no is not None or not suite.uploadable:
-                raise mini_buildd.HTTPBadRequest(f"Distribution '{dist.get()}' not uploadable")
-
-            if not distribution.mbd_is_active():
-                raise mini_buildd.HTTPUnavailable(f"Distribution '{dist.get()}' not active")
-
-            if not self.mbd_is_active():
-                raise mini_buildd.HTTPUnavailable(f"Repository '{self.identity}' not active")
-
-        return distribution, suite
-
 
 def get_meta_distribution_map():
     """Get a dict of the meta distributions: meta -> actual."""
@@ -539,19 +521,19 @@ def get_meta_distribution_map():
         for d in r.distributions.all():
             for s in r.layout.suiteoption_set.all():
                 for m in r.mbd_get_meta_distributions(d, s):
-                    distribution = r.mbd_get_distribution_string(d, s)
+                    diststr = r.mbd_get_diststr(d, s)
                     if m in result:
-                        raise mini_buildd.HTTPInternal(f"Ambiguous Meta-Distributions ({m}={distribution} or {result[m]}). "
+                        raise mini_buildd.HTTPInternal(f"Ambiguous Meta-Distributions ({m}={diststr} or {result[m]}). "
                                                        f"Please check Repositories and Layouts (see Layouts/Meta-Distributions in Administrators Manual).")
-                    result[m] = distribution
+                    result[m] = diststr
 
     LOG.debug(f"Got meta distribution map: {result}")
     return result
 
 
-def map_distribution(dist_str):
+def map_distribution(diststr):
     """Map incoming distribution to internal."""
-    return get_meta_distribution_map().get(dist_str, dist_str)
+    return get_meta_distribution_map().get(diststr, diststr)
 
 
 def get(identity):
@@ -564,9 +546,27 @@ def get(identity):
 
 def parse_dist(dist, check_uploadable=False):
     repository = get(dist.repository)
-    return (repository,) + repository.mbd_parse_dist(dist, check_uploadable=check_uploadable)
+
+    distribution = repository.distributions.all().filter(base_source__codename__exact=dist.codename).first()
+    if distribution is None:
+        raise mini_buildd.HTTPBadRequest(f"No distribution for codename '{dist.codename}' in repository '{repository.identity}'")
+
+    suite = repository.layout.suiteoption_set.filter(suite__name=dist.suite).first()
+    if suite is None:
+        raise mini_buildd.HTTPBadRequest(f"No distribution for suite '{dist.suite}' in repository '{repository.identity}'")
+
+    if check_uploadable:
+        if dist.rollback_no is not None or not suite.uploadable:
+            raise mini_buildd.HTTPBadRequest(f"Distribution '{dist.get()}' not uploadable")
+
+        if not distribution.mbd_is_active():
+            raise mini_buildd.HTTPUnavailable(f"Distribution '{dist.get()}' not active")
+
+        if not repository.mbd_is_active():
+            raise mini_buildd.HTTPUnavailable(f"Repository '{repository.identity}' not active")
+
+    return repository, distribution, suite
 
 
 def parse_diststr(diststr, check_uploadable=False):
-    dist = mini_buildd.dist.Dist(diststr)
-    return (dist,) + parse_dist(dist, check_uploadable=check_uploadable)
+    return parse_dist(mini_buildd.dist.Dist(diststr), check_uploadable=check_uploadable)
