@@ -172,30 +172,32 @@ class TemplatePackage(mini_buildd.misc.TmpDir):
     """Copies a package template into a temporary directory (under 'package/')."""
 
     def __init__(self, template):
-        super().__init__()
+        super().__init__(prefix="template-")
         self.template = template
-        self.package_dir = os.path.join(self.tmpdir.name, "package")
+        self.package_dir = os.path.join(self.tmpdir, "package")
         shutil.copytree(os.path.join(mini_buildd.config.PACKAGE_TEMPLATES, template), self.package_dir)
         self.package_version = DebianVersion.stamp()
         self.package_name = None
 
     @property
     def dsc(self):
-        return glob.glob(os.path.join(self.tmpdir.name, "*.dsc"))[0]
+        return glob.glob(os.path.join(self.tmpdir, "*.dsc"))[0]
 
 
 class KeyringPackage(TemplatePackage):
-    def __init__(self, model):
+    def __init__(self):
         super().__init__("archive-keyring")
+        model = mini_buildd.get_daemon().model
+        gnupg = mini_buildd.get_daemon().gnupg
 
-        self.key_id = model.mbd_gnupg().get_first_sec_key()
+        self.key_id = gnupg.get_first_sec_key()
         LOG.debug(f"KeyringPackage using key: '{self.key_id}'")
 
         self.package_name = f"{model.identity}-archive-keyring"
         self.environment = mini_buildd.call.taint_env(
             {"DEBEMAIL": model.email_address,
              "DEBFULLNAME": model.mbd_fullname(),
-             "GNUPGHOME": model.mbd_gnupg().home})
+             "GNUPGHOME": gnupg.home})
 
         # Replace %ID%, %MAINT% and %KEY_ID% in all files in package dir.
         for root, _dirs, files in os.walk(self.package_dir):
@@ -210,7 +212,7 @@ class KeyringPackage(TemplatePackage):
                 os.rename(new_file, old_file)
 
         # Export public GnuPG key into the package
-        model.mbd_gnupg().export(os.path.join(self.package_dir, self.package_name + ".gpg"), identity=self.key_id)
+        gnupg.export(os.path.join(self.package_dir, self.package_name + ".gpg"), identity=self.key_id)
 
         # Generate sources.lists
         for r in mini_buildd.mdls().repository.Repository.objects.all():
@@ -244,7 +246,7 @@ class KeyringPackage(TemplatePackage):
         mini_buildd.call.Call(["dpkg-source",
                                "-b",
                                "package"],
-                              cwd=self.tmpdir.name,
+                              cwd=self.tmpdir,
                               env=self.environment).check()
 
 
@@ -261,13 +263,13 @@ class TestPackage(TemplatePackage):
             mini_buildd.call.Call(["debchange",
                                    f"MINI_BUILDD_OPTION: auto-ports={','.join(auto_ports)}"],
                                   cwd=self.package_dir).check()
-        mini_buildd.call.Call(["dpkg-source", "-b", "package"], cwd=self.tmpdir.name).check()
+        mini_buildd.call.Call(["dpkg-source", "-b", "package"], cwd=self.tmpdir).check()
 
 
 _DEFAULT_PORT_OPTIONS = ["lintian-mode=ignore"]
 
 
-def _port(dsc_url, package, diststr, version, comments=None, options=None, allow_unauthenticated=False):
+def _port(dsc_url, source, diststr, version, comments=None, options=None, allow_unauthenticated=False):
     def changelog_entries():
         """Merge comments and options into plain changelog_entries."""
         changelog_entries = [] if comments is None else comments
@@ -275,19 +277,20 @@ def _port(dsc_url, package, diststr, version, comments=None, options=None, allow
             changelog_entries.append(f"{mini_buildd.changes.Upload.Options.KEYWORD}: {o}")
         return changelog_entries
 
-    with mini_buildd.misc.TemporaryDirectory(prefix="port-") as tmpdir:
-        dfn = mini_buildd.files.DebianName(package, version)
+    with mini_buildd.misc.tmp_dir(prefix="port-") as tmpdir:
+        dfn = mini_buildd.files.DebianName(source, version)
         env = mini_buildd.call.taint_env({"DEBEMAIL": mini_buildd.get_daemon().model.email_address,
                                           "DEBFULLNAME": mini_buildd.get_daemon().model.mbd_fullname(),
-                                          "GNUPGHOME": mini_buildd.get_daemon().model.mbd_gnupg().home})
+                                          "GNUPGHOME": mini_buildd.get_daemon().gnupg.home})
 
         # Download DSC via dget.
         dget = ["dget", "--download-only"]
         if allow_unauthenticated:
             dget.append("--allow-unauthenticated")
-        mini_buildd.call.Call(dget + [dsc_url],
-                              cwd=tmpdir,
-                              env=env).check()
+        try:
+            mini_buildd.call.Call(dget + [dsc_url], cwd=tmpdir, env=env).check()
+        except Exception as e:
+            raise mini_buildd.HTTPBadRequest("Given URL can't be downloaded (maybe retry allowing unauthenticated)") from e
 
         # Get SHA1 of original dsc file
         original_dsc_sha1sum = mini_buildd.misc.Hash(os.path.join(tmpdir, os.path.basename(dsc_url))).sha1()
@@ -329,7 +332,7 @@ def _port(dsc_url, package, diststr, version, comments=None, options=None, allow
         # Repack DST
         mini_buildd.call.Call(["dpkg-source", "-b", dst], cwd=tmpdir, env=env).check()
         dsc = os.path.join(tmpdir, dfn.dsc())
-        mini_buildd.get_daemon().model.mbd_gnupg().sign(dsc)
+        mini_buildd.get_daemon().gnupg.sign(dsc)
 
         # Gen changes file name
         changes = os.path.join(tmpdir, dfn.changes("source"))
@@ -347,46 +350,54 @@ def _port(dsc_url, package, diststr, version, comments=None, options=None, allow
                                   stdout=out).check()
 
         # Sign and upload
-        mini_buildd.get_daemon().model.mbd_gnupg().sign(changes)
+        mini_buildd.get_daemon().gnupg.sign(changes)
         mini_buildd.changes.Base(changes).upload(mini_buildd.get_daemon().model.mbd_get_ftp_endpoint(), force=True)
-        return diststr, package, version
+        return diststr, source, version
 
 
-def port(package, from_diststr, to_diststr, version, options=None):
+def port(source, from_diststr, to_diststr, version, options=None):
     # check from_diststr
-    _from_dist, from_repository, from_distribution, from_suite = mini_buildd.models.repository.parse_diststr(from_diststr)
-    p = from_repository.mbd_reprepro.find(package, distribution=from_diststr, version=version)
+    from_repository, from_distribution, from_suite = mini_buildd.mdls().repository.parse_diststr(from_diststr)
+    from_repository.mbd_reprepro.ls(source).filter(diststr=from_diststr, version=version, raise_if_not_found="Source to port not found")
 
     # check to_diststr
-    _to_dist, to_repository, to_distribution, to_suite = mini_buildd.models.repository.parse_diststr(to_diststr, check_uploadable=True)
+    to_repository, to_distribution, to_suite = mini_buildd.mdls().repository.parse_diststr(to_diststr, check_uploadable=True)
 
     # Ponder version to use
-    v = DebianVersion(p["version"])
+    v = DebianVersion(version)
     if to_diststr == from_diststr:
         port_version = v.gen_internal_rebuild()
     else:
         port_version = v.gen_internal_port(from_repository.mbd_get_mandatory_version_regex(from_distribution, from_suite),
                                            to_repository.mbd_get_default_version(to_distribution, to_suite))
 
-    _component, path = from_repository.mbd_find_dsc_path(from_distribution, package, p["version"])
-    if not path:
-        raise mini_buildd.HTTPBadRequest(f"Can't find DSC for {package}-{p['version']} in pool")
-
-    return _port(mini_buildd.config.URIS["repositories"]["static"].url_join(path), package, to_diststr, port_version, options=options, allow_unauthenticated=True)
+    return _port(mini_buildd.config.URIS["repositories"]["static"].url_join(from_repository.mbd_dsc_pool_path(source, version)),
+                 source,
+                 to_diststr,
+                 port_version,
+                 options=options,
+                 allow_unauthenticated=True)
 
 
 def port_ext(dsc_url, to_diststr, options=None, allow_unauthenticated=False):
-    _to_dist, to_repository, to_distribution, to_suite = mini_buildd.models.repository.parse_diststr(to_diststr, check_uploadable=True)
-    with urllib.request.urlopen(dsc_url) as response:
-        dsc = debian.deb822.Dsc(response)
-        v = DebianVersion(dsc["Version"])
-        return _port(dsc_url,
-                     dsc["Source"],
-                     to_diststr,
-                     v.gen_external_port(to_repository.mbd_get_default_version(to_distribution, to_suite)),
-                     comments=[f"External port from: {dsc_url}"],
-                     options=options,
-                     allow_unauthenticated=allow_unauthenticated)
+    to_repository, to_distribution, to_suite = mini_buildd.mdls().repository.parse_diststr(to_diststr, check_uploadable=True)
+    try:
+        with urllib.request.urlopen(dsc_url) as response:
+            dsc = debian.deb822.Dsc(response)
+            source = dsc.get("Source")
+            if source is None:
+                raise mini_buildd.HTTPBadRequest("Given URL not a DSC")
+
+            v = DebianVersion(dsc["Version"])
+            return _port(dsc_url,
+                         source,
+                         to_diststr,
+                         v.gen_external_port(to_repository.mbd_get_default_version(to_distribution, to_suite)),
+                         comments=[f"External port from: {dsc_url}"],
+                         options=options,
+                         allow_unauthenticated=allow_unauthenticated)
+    except urllib.error.URLError as e:
+        raise mini_buildd.HTTPBadRequest(f"{e} ({dsc_url})")
 
 
 def upload_template_package(template_package, diststr):
